self.__BUILD_MANIFEST={polyfillFiles:["static/chunks/polyfills-42372ed130431b0a.js"],devFiles:[],ampDevFiles:[],lowPriorityFiles:[],rootMainFiles:["static/chunks/webpack-06c74e0057d02fc8.js","static/chunks/4bd1b696-d6120dc685f4dd62.js","static/chunks/215-5cff7675b32ccaef.js","static/chunks/main-app-3e3c0d43fb1c97f1.js"],rootMainFilesTree:{},pages:{"/_app":["static/chunks/webpack-06c74e0057d02fc8.js","static/chunks/framework-b370f160bb96059c.js","static/chunks/main-43a8bac11a1efa4e.js","static/chunks/pages/_app-037b5d058bd9a820.js"],"/_error":["static/chunks/webpack-06c74e0057d02fc8.js","static/chunks/framework-b370f160bb96059c.js","static/chunks/main-43a8bac11a1efa4e.js","static/chunks/pages/_error-6ae619510b1539d6.js"]},ampFirstPages:[]},self.__BUILD_MANIFEST.lowPriorityFiles=["/static/"+process.env.__NEXT_BUILD_ID+"/_buildManifest.js",,"/static/"+process.env.__NEXT_BUILD_ID+"/_ssgManifest.js"];