"""Defaults used across curator."""

BATCH_REQUEST_ID_TAG = "custom_id"
_CURATOR_DEFAULT_CACHE_DIR = "~/.cache/curator"
_DEFAULT_CACHE_DIR = "~/.cache"
BASE_CLIENT_URL = "https://api.bespokelabs.ai/v0/viewer"
PUBLIC_CURATOR_VIEWER_URL = "https://curator.bespokelabs.ai/datasets"
